package cn.hanghui.outapi.platform.alipay.service.impl;

import cn.hanghui.outapi.platform.alipay.api.AlipayCommonApi;
import cn.hanghui.outapi.platform.alipay.api.AlipayUserApi;
import cn.hanghui.outapi.platform.alipay.entity.dto.AlipayUserInfoExtInfoReqDto;
import cn.hanghui.outapi.platform.alipay.entity.dto.AlipayUserInfoReqDto;
import cn.hanghui.outapi.platform.alipay.entity.dto.GaAlipayUserInfoBizPackageReq;
import cn.hanghui.outapi.platform.alipay.entity.dto.GaAlipayUserInfoReq;
import cn.hanghui.outapi.platform.alipay.entity.param.AlipayUserInfo;
import cn.hanghui.outapi.platform.alipay.service.AlipayFaceAuthService;
import cn.hanghui.outapi.platform.alipay.service.AlipayOpenApiService;
import cn.hanghui.outapi.platform.common.service.BaseService;
import cn.hanghui.outapi.platform.common.utils.CipherUtil;
import cn.hanghui.outapi.platform.common.utils.JSONUtil;
import cn.hanghui.outapi.platform.common.utils.Preconditions;
import cn.hanghui.outapi.platform.common.utils.SM2KeyUtil;
import cn.hanghui.outapi.platform.common.utils.SnowflakeIdWorker;
import cn.hanghui.outapi.platform.tenant.entity.dto.TenantVo;
import cn.hanghui.outapi.platform.tenant.service.TenantService;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

@Service
@Slf4j
public class AlipayFaceAuthServiceImpl extends BaseService implements AlipayFaceAuthService {

    @Resource
    private AlipayUserApi alipayUserApi;

    @Resource
    private AlipayClient alipayFaceAuthClient;

    @Resource
    private AlipayClient alipayOpenApiClient5513;

    @Resource
    private AlipayCommonApi alipayCommonApi;

    @Autowired
    private TenantService tenantService;

    @Autowired
    private AlipayOpenApiService alipayOpenApiService;


    /**
     * 获取支付宝用户信息
     *
     * @param alipayUserInfoDto AlipayUserInfoDto
     * @return AlipayUserInfo
     */
    @Override
    public AlipayUserInfo getAlipayUserInfo(AlipayUserInfoReqDto alipayUserInfoDto) {
        /**---------------------------以下逻辑为新版本兼容老版本逻辑-------------------*/
        AlipayUserInfo userInfo = null;
        AlipayClient alipayClient = alipayFaceAuthClient;
        //根据sn获取三方授权商户信息（auth_token/auth_code）
        String thirdAppAuthToken = null;
        TenantVo tenant = tenantService.getThirdAppAuthTokenBySn(alipayUserInfoDto.getSn());
        if (tenant != null) {
            alipayClient = alipayOpenApiClient5513;
            thirdAppAuthToken = tenant.getAppAuthToken();
        }

        //兼容老版本
        if (alipayUserInfoDto.getAuthCode() != null && !"".equals(alipayUserInfoDto.getAuthCode().trim())) {
            userInfo = alipayUserApi.getUserInfo(alipayClient, alipayUserInfoDto.getAuthCode(), thirdAppAuthToken);
        }

        //封闭库 vid编码规则 IDENTITY_CARD + 机构编码 + 用户身份证
        if (userInfo == null
                && alipayUserInfoDto.getVid() != null
                && !"".equals(alipayUserInfoDto.getVid().trim())) {
            IsvInfo isvInfo = getIsvInfo(null);
            userInfo = alipayOpenApiService.getUserInfo(alipayUserInfoDto.getVid(), isvInfo.getOrgOutId());
        }

        //通用
        AlipayUserInfoExtInfoReqDto extInfoReqDto = handleExtInfo(alipayUserInfoDto.getExtInfo());
        if (userInfo == null
                && extInfoReqDto != null
                && extInfoReqDto.getAuthCode() != null
                && !"".equals(extInfoReqDto.getAuthCode().trim())) {
            //获取支付宝用户三要素+头像
            userInfo = alipayUserApi.getUserInfo(alipayClient, extInfoReqDto.getAuthCode(), thirdAppAuthToken);
        }

        //添加新逻辑 刷身份证
        if (userInfo == null
                && alipayUserInfoDto != null
                && alipayUserInfoDto.getCardnumber() != null
                && alipayUserInfoDto.getCardname() != null) {
            //刷身份证
            userInfo = new AlipayUserInfo();
            userInfo.setUserName(alipayUserInfoDto.getCardname());
            userInfo.setCertNo(alipayUserInfoDto.getCardnumber());
        }

        if (userInfo == null) {
            userInfo = new AlipayUserInfo();
        }

//        //获取现场刷脸照片 目前只有贵阳机场支持
//        if (alipayUserInfoDto.getFtoken() != null && !"".equals(alipayUserInfoDto.getFtoken().trim())) {
//            String photoBase64 = alipayUserApi.getPhotoBase64ByFtoken(alipayClient, alipayUserInfoDto.getFtoken(), "2", thirdAppAuthToken, alipayUserInfoDto.getAlipayUid());
//            userInfo.setPhotoBase64(photoBase64);
//        }
        log.info("--->获取支付宝用户信息响应参数:{}", JSONObject.toJSONString(userInfo));
        return userInfo;
    }


    @Override
    public void authIsvCallback(String appId, String appAuthCode) {
        try {
            //获取授权accessToken
            String accessToken = alipayCommonApi.getOpenAuthToken(alipayOpenApiClient5513, appAuthCode);
            if (accessToken == null) {
                return;
            }
            //根据appId更新accessToken
//            tenantService.updateAppAccessTokenByAppId(appId, accessToken);
        } catch (Exception e) {
            log.error("--->三方授权回调获取token服务异常", e);
        }

    }

    @Override
    public AlipayUserInfo getCommonAlipayUserInfo(AlipayUserInfoReqDto alipayUserInfoDto) {
        /**---------------------------以下逻辑为新版本兼容老版本逻辑-------------------*/
        AlipayUserInfo userInfo = null;
        String appId = null;

        if (alipayUserInfoDto.getAppId() != null
                && !"".equals(alipayUserInfoDto.getAppId().trim())) {
            appId = alipayUserInfoDto.getAppId();
        }

        if (Preconditions.isBlank(appId) && alipayUserInfoDto.getSn() != null && !"".equals(alipayUserInfoDto.getSn().trim())) {
            TenantVo tenantVo = tenantService.getBySn(alipayUserInfoDto.getSn());
            if (tenantVo != null) {
                appId = tenantVo.getServiceAppId();
            }
        }

        AlipayClient alipayClient = getAlipayClient(appId);
        //兼容老版本
        if (alipayUserInfoDto.getAuthCode() != null && !"".equals(alipayUserInfoDto.getAuthCode().trim())) {
            userInfo = alipayUserApi.getUserInfo(alipayClient, alipayUserInfoDto.getAuthCode(), null);
        }

        //封闭库 vid编码规则 IDENTITY_CARD + 机构编码 + 用户身份证
        if (userInfo == null
                && alipayUserInfoDto.getVid() != null
                && !"".equals(alipayUserInfoDto.getVid().trim())) {
            IsvInfo isvInfo = getIsvInfo(appId);
            userInfo = alipayOpenApiService.getUserInfo(alipayUserInfoDto.getVid(), isvInfo.getOrgOutId());
        }

        //通用
        AlipayUserInfoExtInfoReqDto extInfoReqDto = handleExtInfo(alipayUserInfoDto.getExtInfo());
        if (userInfo == null
                && extInfoReqDto != null
                && extInfoReqDto.getAuthCode() != null
                && !"".equals(extInfoReqDto.getAuthCode().trim())) {
            //获取支付宝用户三要素+头像
            userInfo = alipayUserApi.getUserInfo(alipayClient, extInfoReqDto.getAuthCode(), null);
        }

        //添加新逻辑 刷身份证
        if (userInfo == null
                && alipayUserInfoDto != null
                && alipayUserInfoDto.getCardnumber() != null
                && alipayUserInfoDto.getCardname() != null) {
            //刷身份证
            userInfo = new AlipayUserInfo();
            userInfo.setUserName(alipayUserInfoDto.getCardname());
            userInfo.setCertNo(alipayUserInfoDto.getCardnumber());
        }

        if (userInfo == null) {
            userInfo = new AlipayUserInfo();
        }

        //获取现场刷脸照片 目前只有贵阳机场/3138支持
        if (alipayUserInfoDto.getFtoken() != null && !"".equals(alipayUserInfoDto.getFtoken().trim()) && checkFtoken(appId)) {
            String photoBase64 = alipayUserApi.getPhotoBase64ByFtoken(alipayClient, alipayUserInfoDto.getFtoken(), "2", null, alipayUserInfoDto.getAlipayUid());
            userInfo.setPhotoBase64(photoBase64);
        }
        log.info("--->获取支付宝用户信息响应参数:{}", JSONObject.toJSONString(userInfo));
        return userInfo;
    }

    /**
     * 数据转换
     *
     * @param extInfo
     * @return
     */
    private AlipayUserInfoExtInfoReqDto handleExtInfo(Object extInfo) {
        AlipayUserInfoExtInfoReqDto dto = null;
        try {
            if (extInfo instanceof String) {
                dto = JSONObject.parseObject(String.valueOf(extInfo), AlipayUserInfoExtInfoReqDto.class);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("--->extInfo数据转换异常:{}", extInfo);

        }
        return dto;
    }

    @Override
    public AlipayUserInfo queryCommonUserInfo(AlipayUserInfoReqDto alipayUserInfoDto) {
        /**---------------------------以下逻辑为新版本兼容老版本逻辑-------------------*/
        AlipayUserInfo userInfo = null;
        String appId = null;

        if (alipayUserInfoDto.getAppId() != null
                && !"".equals(alipayUserInfoDto.getAppId().trim())) {
            appId = alipayUserInfoDto.getAppId();
        }

        if (Preconditions.isBlank(appId) && alipayUserInfoDto.getSn() != null && !"".equals(alipayUserInfoDto.getSn().trim())) {
            TenantVo tenantVo = tenantService.getBySn(alipayUserInfoDto.getSn());
            if (tenantVo != null) {
                appId = tenantVo.getServiceAppId();
            }
        }

        AlipayClient alipayClient = getAlipayClient(appId);
        //兼容老版本
        if (alipayUserInfoDto.getAuthCode() != null && !"".equals(alipayUserInfoDto.getAuthCode().trim())) {
            userInfo = alipayUserApi.getUserInfo(alipayClient, alipayUserInfoDto.getAuthCode(), null);
        }

        //封闭库 vid编码规则 IDENTITY_CARD + 机构编码 + 用户身份证
        if (userInfo == null
                && alipayUserInfoDto.getVid() != null
                && !"".equals(alipayUserInfoDto.getVid().trim())) {
            userInfo = alipayOpenApiService.getUserInfo(alipayUserInfoDto.getVid());
        }

        //通用
        AlipayUserInfoExtInfoReqDto extInfoReqDto = handleExtInfo(alipayUserInfoDto.getExtInfo());
        ;
        if (userInfo == null
                && extInfoReqDto != null
                && extInfoReqDto.getAuthCode() != null
                && !"".equals(extInfoReqDto.getAuthCode().trim())) {
            //获取支付宝用户三要素+头像
            userInfo = alipayUserApi.getUserInfo(alipayClient, extInfoReqDto.getAuthCode(), null);
        }

        //添加新逻辑 刷身份证
        if (userInfo == null
                && alipayUserInfoDto != null
                && alipayUserInfoDto.getCardnumber() != null
                && alipayUserInfoDto.getCardname() != null) {
            //刷身份证
            userInfo = new AlipayUserInfo();
            userInfo.setUserName(alipayUserInfoDto.getCardname());
            userInfo.setCertNo(alipayUserInfoDto.getCardnumber());
        }

        if (userInfo == null) {
            userInfo = new AlipayUserInfo();
        }

        //获取现场刷脸照片 目前只有贵阳机场支持
        if (alipayUserInfoDto.getFtoken() != null && !"".equals(alipayUserInfoDto.getFtoken().trim()) && checkFtoken(appId)) {
            String photoBase64 = alipayUserApi.getPhotoBase64ByFtoken(alipayClient, alipayUserInfoDto.getFtoken(), "2", null, alipayUserInfoDto.getAlipayUid());
            userInfo.setPhotoBase64(photoBase64);
        }
        log.info("--->获取支付宝用户信息响应参数:{}", JSONObject.toJSONString(userInfo));
        return userInfo;
    }

    @Value("${ga.alipayUserInfoUrl}")
    private String alipayUserInfoUrl;
    @Value("${ga.seid}")
    private String seid;
    @Value("${ga.bizType}")
    private String bizType;
    @Value("${ga.signKey}")
    private String signKey;
    @Value("${ga.verifySignKey}")
    private String verifySignKey;
    @Value("${ga.signKey}")
    private String encryptKey;


    @Override
    public AlipayUserInfo queryGaUserInfo(AlipayUserInfoReqDto alipayUserInfoReqDto) {
        GaAlipayUserInfoReq alipayUserInfoReq = new GaAlipayUserInfoReq();
        //bizPackage组装业务参数
        GaAlipayUserInfoBizPackageReq bizPackage = BeanUtil.copyProperties(alipayUserInfoReqDto, GaAlipayUserInfoBizPackageReq.class);
        bizPackage.setRequestId(DigestUtil.md5Hex(SnowflakeIdWorker.getSnowId() + UUID.randomUUID().toString().replace("-", "")));
        bizPackage.setRequestTime(String.valueOf(System.currentTimeMillis()));
        bizPackage.setSeid(seid);
        bizPackage.setBizType(bizType);
        alipayUserInfoReq.setBizPackage(JSONObject.toJSONString(bizPackage));
        //签名
        String body = JSONUtil.toJson(bizPackage);
        String sign = doSignature(body);
        alipayUserInfoReq.setSign(sign);

        // 添加请求头信息
        Map<String, String> heads = new HashMap<>();
        // 使用json发送请求，下面的是必须的
        heads.put("Content-Type", "application/json;charset=UTF-8");
        heads.put("gp_sign", sign);
        HttpResponse resp = HttpRequest.post(alipayUserInfoUrl)
                .headerMap(heads, false)
                .body(String.valueOf(body))
                .timeout(5 * 60 * 1000)
                .execute();
        log.info("--->获取公安一所支付宝用户信息响应参数：{}", JSONObject.toJSONString(resp));
        GaAlipayUserInfoReq userInfoResq = JSONObject.parseObject(resp.body(), GaAlipayUserInfoReq.class);
        userInfoResq.getBizPackage();
        CipherUtil.decrypt();


        return null;
    }

    /**
     * 接口请求数据签名
     *
     * @param params 1
     * @return java.lang.String
     * @author Gaoch
     * @since 2022/04/20 10:48:13
     */
    private String doSignature(String... params) {
        String content = String.join(";", params);
        log.info("签名原文:" + content);
        return CipherUtil.sign(signKey, content);
    }


}
